import Helper from "../utils/helper";
import { EndPoint, LangSlug, } from "../utils/interfaces";
import Submission from "./submission";
import * as cherrio from "cheerio";
class Problem {
    slug;
    id;
    title;
    categoryTitle;
    difficulty;
    starred;
    locked;
    likes;
    dislikes;
    status;
    tag;
    totalAccepted;
    totalSubmission;
    hints;
    sampleTestCase;
    exampleTestcases;
    content;
    translatedContent;
    translatedTitle;
    codeSnippets;
    constructor(slug, id, title, categoryTitle, difficulty, starred, locked, likes, dislikes, status, tag, totalAccepted, totalSubmission, hints, sampleTestCase, exampleTestcases, content, translatedContent, translatedTitle, codeSnippets) {
        this.slug = slug;
        this.id = id;
        this.title = title;
        this.categoryTitle = categoryTitle;
        this.difficulty = difficulty;
        this.starred = starred;
        this.locked = locked;
        this.likes = likes;
        this.dislikes = dislikes;
        this.status = status;
        this.tag = tag;
        this.totalAccepted = totalAccepted;
        this.totalSubmission = totalSubmission;
        this.hints = hints;
        this.sampleTestCase = sampleTestCase;
        this.exampleTestcases = exampleTestcases;
        this.content = content;
        this.translatedContent = translatedContent;
        this.translatedTitle = translatedTitle;
        this.codeSnippets = codeSnippets;
    }
    static async build(slug) {
        const problem = new Problem(slug);
        await problem.detail();
        return problem;
    }
    async detail() {
        const response = await Helper.GraphQLRequest({
            query: `
                query getQuestionDetail($titleSlug: String!) {
                    question(titleSlug: $titleSlug) {
                        questionId
                        title
                        categoryTitle
                        difficulty
                        likes
                        dislikes
                        isLiked
                        isPaidOnly
                        stats
                        status
                        content
                        topicTags {
                            name
                            slug
                            ${Helper.endpoint === EndPoint.CN
                ? "translatedName"
                : ""}
                        }
                        codeSnippets {
                            lang
                            langSlug
                            code
                        }
                        hints
                        exampleTestcases
                        sampleTestCase
                        ${Helper.endpoint === EndPoint.CN
                ? "translatedContent"
                : ""}
                        ${Helper.endpoint === EndPoint.CN
                ? "translatedTitle"
                : ""}
                    }
                }
            `,
            variables: {
                titleSlug: this.slug,
            },
        });
        const question = response.question;
        this.id = Number(question.questionId);
        this.title = question.title;
        this.categoryTitle = question.categoryTitle;
        this.difficulty = Helper.difficultyMap(question.difficulty);
        this.starred = question.isLiked !== null;
        this.locked = question.isPaidOnly;
        this.likes = question.likes;
        this.dislikes = question.dislikes;
        this.status = Helper.statusMap(question.status);
        this.tag = question.topicTags;
        const stats = JSON.parse(question.stats);
        this.totalAccepted = stats.totalAcceptedRaw;
        this.totalSubmission = stats.totalSubmissionRaw;
        this.hints = question.hints;
        this.sampleTestCase = question.sampleTestCase;
        this.exampleTestcases = question.exampleTestcases;
        this.content = question.content;
        this.translatedContent = question.translatedContent;
        this.translatedTitle = question.translatedTitle;
        this.codeSnippets = question.codeSnippets;
        return this;
    }
    async getContentImages() {
        const res = {
            content: new Map(),
            translatedContent: new Map(),
        };
        const getImg = async (htmlContent, mp) => {
            const $ = cherrio.load(htmlContent);
            $("img").each((ix, e) => {
                mp.set($(e).attr("src"), "");
            });
            for (const key of mp.keys()) {
                const resp = await Helper.HttpRequest({
                    url: key,
                    encoding: "binary",
                });
                mp.set(key, resp);
            }
        };
        if (this.content) {
            await getImg(this.content, res.content);
        }
        if (this.translatedContent) {
            await getImg(this.translatedContent, res.translatedContent);
        }
        return res;
    }
    async getSubmissions() {
        const submissions = [];
        let offset = 0;
        const limit = 20;
        let hasNext = true;
        while (hasNext) {
            const response = await Helper.GraphQLRequest({
                query: `
                query Submissions($offset: Int!, $limit: Int!, $questionSlug: String!) {
                    submissionList(offset: $offset, limit: $limit, questionSlug: $questionSlug) {
                        lastKey
                        hasNext
                        submissions {
                            id
                            statusDisplay
                            lang
                            runtime
                            timestamp
                            memory
                        }
                    }
                }
                `,
                variables: {
                    offset: offset,
                    limit: limit,
                    questionSlug: this.slug,
                },
            });
            hasNext = response.submissionList.hasNext;
            const submission = response.submissionList.submissions;
            offset += submission.length;
            submission.map((s) => {
                submissions.push(new Submission(Number(s.id), s.lang, s.memory, s.runtime, s.statusDisplay, Number(s.timestamp), undefined, undefined));
            });
        }
        return submissions;
    }
    async submit(langSlug, code) {
        const response = await Helper.HttpRequest({
            url: Helper.uris.submit.replace("$slug", this.slug),
            method: "POST",
            body: {
                lang: LangSlug[langSlug],
                question_id: this.id,
                typed_code: code,
            },
        });
        return new Submission(JSON.parse(response).submission_id);
    }
}
export default Problem;
//# sourceMappingURL=problem.js.map